#################### calculate_feature_density_by_reporting_units.py ######################
# Calculates feature density by reporting units
# For polygons, Density is the percent of reporting occupied by area of polygons
# For lines, Density is the length (km) per area (square km)
# For points, Density is the number per square kilometer
# If features overlap, you must run dissolve or risk potentially double-counting values

# Inputs:
# Input Feature Class
# Reporting Unit Feature Class
# Output Feature Class  (this will be a copy of reporting unit feature class with density field added)
#
#Outputs:
# Output feature class (so it can be consumed by later tools)
# Output field name
############################################################

import arcpy,os

workspace=arcpy.env.scratchWorkspace

inFC=arcpy.GetParameterAsText(0)
dissolveInFC=arcpy.GetParameter(1)
ruFC=arcpy.GetParameterAsText(2)
outFC=arcpy.GetParameterAsText(3)

disFC="%s/temp_dis"%(workspace)
intFC="%s/temp"%(workspace)
for fc in [outFC,disFC,intFC]:
    if arcpy.Exists(fc):
        arcpy.Delete_management(fc)
        
#need to dissolve first or risk double counting overlapping features!
inputFC=inFC
if dissolveInFC:        
    arcpy.AddMessage("Dissolving...")
    arcpy.Dissolve_management(inFC,disFC,"","","SINGLE_PART")
    inputFC=disFC

arcpy.AddMessage("Intersecting with reporting units...")
arcpy.Intersect_analysis ([ruFC,inputFC],intFC, "ONLY_FID", "", "")

FID_att="FID_%s"%(os.path.split(arcpy.mapping.Layer(ruFC).dataSource)[1])

arcpy.AddMessage("Tallying results...")
results=dict()
featureType=None
rows=arcpy.SearchCursor(intFC,"","","SHAPE,%s"%(FID_att)) #get cursor with just shape and fid atts
for row in rows:
    rowID=row.getValue(FID_att)
    if not results.has_key(rowID):
        results[rowID]=0
    if not featureType:
        featureType=row.shape.type
    if featureType.count("polygon"):
        value=row.shape.area
    if featureType.count("line"):
        value=row.shape.length
    if featureType.count("point"):
        value=1  #just count the points
    results[rowID]+=value
del row,rows

arcpy.AddMessage("Adding results to output...")
arcpy.CopyFeatures_management(ruFC,outFC)
arcpy.AddField_management(outFC, "DENSITY", "DOUBLE", 10, 4, "","", "NULLABLE")
rows=arcpy.UpdateCursor(outFC,"","","SHAPE,OBJECTID,DENSITY")
for row in rows:
    rowID=row.getValue("OBJECTID")
    density=0
    area=float(row.shape.area)
    if (results.has_key(rowID)) and area>0:
        if featureType.count("point"):
            #density=points per square kilometer
            density=float(results[rowID])/(area/1000000.0)
        elif featureType.count("line"):
            #density=kilometers per square kilometer
            density=(float(results[rowID]))/(area/1000.0)
        elif featureType.count("polygon"):
            density=100.0 * float(results[rowID])/area 
    row.setValue("DENSITY",density)
    rows.updateRow(row)
del row,rows

#delete intermediate
for fc in [disFC,intFC]:
    if arcpy.Exists(fc):
        arcpy.Delete_management(fc)

arcpy.SetParameter(3,outFC)
arcpy.SetParameter(4,"DENSITY")




