########################################################################
# EEMSNode.py

# EEMS: Environmental Evaluation Modeling System

# A modeling system based very closely on EMDS. Allows for implementation
# of logic models based primarily on fuzzy logic.

# Script by Tim Sheehan, with portions based on work done by Brendan Ward

# Tim Sheehan
# Ecological Modeler
# Conservation Biology Institute
# www.consbio.org

# The overall idea behind EEMS in ArcGIS is

# Create an input/results table, feature class, or shapefile 
#   with input values

# Implement the desired logic model using EEMS commands in model
#   builder, fed by the input/results table. This creates a .eem
#   command file.

# Using the EEMS model run tool, use the .eem file and the input/
# results table to run the model and store the results in the
# input/results file.

# This script, EEMSNode.py, is the script used by the EEMS commands
# to generate lines in the .eem file.

# Dev History:

# 2012.02.24 Model Builder command interfaces built,
#   and individually tested.
#
########################################################################

import arcpy,os,csv,re,time # arc library
from arcpy.sa import *
from arcpy import env

########################################################################
# Globals for control, etc
DEBUG = True
MODELBUILDER = True
########################################################################

def OutMsg(msg):
    if MODELBUILDER:
        arcpy.AddMessage('\t%s'%(msg))
    else:
        print(msg);
# def OutMsg(msg):

def ProfileTime(outStr):
    OutMsg("%7.3f %s"%(time.clock(),outStr))
# def ProfileTime(outStr):

def RaiseException(errStr):
    try:
        raise Exception(errStr)
    except Exception as inst:
        OutMsg(errStr)
    raise
# end def RaiseException(errStr):

def BuildNodeLine(cmd,outFldNm,inFldNms,numerics):

    specialCmds = ['READ','CVTTOFUZZY','SELECTEDUNION']
    argSglFldCmds = ['COPYFIELD','NOT']
    argFldsOnlyCmds = ['OR','ORNEG','XOR','SUM','DIF','MIN','MAX','MEAN','UNION','AND']
    argFldsAndWtsCmds = ['WTDUNION','WTDAND','WTDMEAN','WTDSUM']

    if inFldNms == '':
        lInFldNms = []
    else:
        lInFldNms = inFldNms.split(';')

    if numerics == '':
        lNumerics = []
    else:
        lNumerics = numerics.split(';')

    if cmd == 'READ':
        if len(lInFldNms)!= 0 or len(lNumerics) != 0:
            RaiseException('ERROR: %s command takes no input fields and no numerics'%(cmd))
        lRtrnLine = '%s:%s'%(outFldNm,cmd)

    elif cmd == 'CVTTOFUZZY':
        if len(lInFldNms) != 1 or len(lNumerics) != 2:
            RaiseException('ERROR: %s command takes 1 input fields and 2 numerics'%(cmd))
        lRtrnLine = '%s:%s:%s:%s'%(outFldNm,cmd,','.join(lInFldNms),','.join(lNumerics))

    elif cmd == 'SELECTEDUNION':
        if len(lInFldNms) == 1 or len(lNumerics) != 2:
            RaiseException('ERROR: %s command takes 1 or more input fields and 2 numerics'%(cmd))
        lRtrnLine = '%s:%s:%s:%s'%(outFldNm,cmd,','.join(lInFldNms),','.join(lNumerics))

    elif cmd in argSglFldCmds:
        if len(lInFldNms)!= 1 or len(lNumerics) != 0:
            RaiseException('ERROR: %s command takes 1 input field and no numerics'%(cmd))
        lRtrnLine = '%s:%s:%s'%(outFldNm,cmd,','.join(lInFldNms))

    elif cmd in argFldsOnlyCmds:
        if len(lInFldNms) == 0 or len(lNumerics) != 0:
            RaiseException("ERROR: %s requiress at least 1 input field and no numerics"%(cmd))
        lRtrnLine = '%s:%s:%s'%(outFldNm,cmd,','.join(lInFldNms))

    elif cmd in argFldsAndWtsCmds:
        if len(lInFldNms) == 0:
            RaiseException("ERROR: %s requiress at least 1 input field"%(cmd))
        if len(lNumerics) > 0 and len(lNumerics) != len(lInFldNms):
            RaiseException("ERROR: %s requiress equal number of fields and  numerics"%(cmd))
        if len(lNumerics) == 0:
            lNumerics = ['1'] * len(lInFldNms)
            
        lRtrnLine = '%s:%s:%s:%s'%(outFldNm,cmd,','.join(lInFldNms),','.join(lNumerics))

    else:
        RaiseException("ERROR: Illegal command: %s"%(cmd))

    return lRtrnLine

    # if cmd in argSglFldsCmds:
# def BuildNodeLine(cmd,inFldNms,numerics,outFldNm):

def WriteLineToFile(fNm,line):
    lOutFile = open(fNm,'a')
    lOutFile.write('%s\n'%(line))
    lOutFile.close()
# def WriteLineToFile(fNm,line):

########################################################################
# Processing starts here
########################################################################

# Script is designed to interface with different Arc dialog boxes,
# with different numbers of arguments, depending on the command

cmd = arcpy.GetParameterAsText(0)

inFldNms = ''
numericArgs = ''

if cmd == 'READ':
    outFldNm = arcpy.GetParameterAsText(2)
    cmdFileNm = arcpy.GetParameterAsText(3)
    outNdx = 4
elif cmd == 'CVTTOFUZZY':
    inFldNms = arcpy.GetParameterAsText(1)
    falseThresh = arcpy.GetParameterAsText(2)
    trueThresh = arcpy.GetParameterAsText(3)
    numericArgs = falseThresh + ';' + trueThresh
    outFldNm = arcpy.GetParameterAsText(4)
    if outFldNm == 'DEFAULT':
        outFldNm = inFldNms + 'Fz'
    cmdFileNm = arcpy.GetParameterAsText(5)
    outNdx = 6

elif cmd == 'SELECTEDUNION':
    inFldNms = arcpy.GetParameterAsText(1)
    numToSelect = arcpy.GetParameterAsText(2)
    trueOrFalse = arcpy.GetParameterAsText(3)
    if trueOrFalse.lower() == 'truest':
        trueOrFalse = '1'
    elif trueOrFalse.lower() == 'falsest':
        trueOrFalse = '-1'
    else:
        RaiseException('SELECTEDUNION command has illegal True/False spec: **%s**'%(trueOrFalse))
    numericArgs = ';'.join((trueOrFalse,numToSelect))
    # if trueOrFalse.lower() == 'true':
    outFldNm = arcpy.GetParameterAsText(4)
    cmdFileNm = arcpy.GetParameterAsText(5)
    outNdx = 6
elif cmd in ['COPYFIELD','NOT']:
    inFldNms = arcpy.GetParameterAsText(1)
    outFldNm = arcpy.GetParameterAsText(2)
    cmdFileNm = arcpy.GetParameterAsText(3)
    outNdx = 4
elif cmd in ['OR','ORNEG','XOR','SUM','MIN','MAX','MEAN','UNION','AND']:
    inFldNms = arcpy.GetParameterAsText(1)
    outFldNm = arcpy.GetParameterAsText(2)
    cmdFileNm = arcpy.GetParameterAsText(3)
    outNdx = 4
elif cmd in ['DIF']:
    inFldNms = arcpy.GetParameterAsText(1) + ';' + arcpy.GetParameterAsText(2)
    outFldNm = arcpy.GetParameterAsText(3)
    cmdFileNm = arcpy.GetParameterAsText(4)
    outNdx = 5
elif cmd in ['WTDUNION','WTDAND','WTDMEAN','WTDSUM']:
    inFldNms = arcpy.GetParameterAsText(1)
    numericArgs = arcpy.GetParameterAsText(2)
    outFldNm = arcpy.GetParameterAsText(3)
    cmdFileNm = arcpy.GetParameterAsText(4)
    outNdx = 5
else:
    RaiseException('Illegal Command: %s'%(cmd))
# if cmd == 'READ':...elif...else

if DEBUG:
    OutMsg('cmd %s'%(cmd))
    OutMsg('inFldNms %s'%(inFldNms))
    OutMsg('numericArgs %s'%(numericArgs))
    OutMsg('outFldNm %s'%(outFldNm))
    OutMsg('cmdFileNm %s'%(cmdFileNm))

line = BuildNodeLine(cmd,outFldNm,inFldNms,numericArgs)
WriteLineToFile(cmdFileNm,line)

arcpy.AddMessage("\tAdded Line to %s:\n\t\t%s"%(cmdFileNm,line))

# RaiseException('stop')
arcpy.SetParameter(outNdx,outFldNm)


